%analysis tool for fitting van-genuchten parameters from input data
%from ku-pf-apparatus (UGT Mncheberg)

%(GNU) creative commons:        Conrad Jackisch, 2006
%http://creativecommons.org/    Institute for Geoecology, 
%                               University Potsdam, Germany

%take care to prepare the input data as the sample files kupf_X.txt and
%kupf_Xhead.txt, more analysis from the data is possible, but this version
%will cover the van genuchten parameter estimation only.

%this script is open for use and change without any warranty as GNU GPL
%####################################

%OBJECTIVE: simple tool for van genuchten parameter fitting; the fitting is
%           weighed according to even distribution of data posts for all pF
%           hence the high data density will not rule out the much more
%           precise data from the pressure pots.

%INPUT: DATA SERIES FROM KU-PF APPARATUS
%       DATA FROM PRESSURE POTS
%       FURTHER SOIL PROPERTIES

%OUTPUT:    FITTED VAN-GENUCHTEN PARAMETERS
%           TO DESCRIBE UNSATURATED CONDUCTIVITY

%####################################
%DEFINITIONS:
%soil water content:    theta = V(water)/V(soil) = m(water)/m(drysoil) * BD [density of water=1]
%soil water retention:  theta = theta_res + (theta_sat-theta_res)/(1+(alpha*h)^n)^m  >> van genuchten
%hydraulic conductivity:K(theta)/Ksat = sqrt(theta - theta(resid)/(por - theta(resid)) * (1 - (1 - (theta - theta(resid)/(por - theta(resid))^(1/m))^m)^2 >> van genuchten
%with: alpha=1/(hb)
%with: n=lambda+1
%with: m=lambda/n


clear
%####################################
%PARAMETERS:
datafile=['conrad_1m.txt'];
headerfile=['conrad_1head.txt'];

%time_step=600;%timestep in seconds

%fitting criteria
fit=50;%sum of squared residuals after fitting
fit2=10;%difference of errorfitting between two steps to break
tuningn=0.005;
tuninga=0.002;

%initial van genuchten paramters

%!! It makes sense to use e.g. Rosetta before using this fitting routine !!
%!! The initial values proved to be able to influence the result according
%   to the given fitting criteria. Rosetta can be found at:
%   http://www.ars.usda.gov/Services/docs.htm?docid=8953

K_ini=0.14;%initial Ksat
n_ini=1.54;%initial n
alpha_ini=0.009;%initial alpha

%soil properties
por=0.36;%estimated total porosity

%constants
Aopen=41;%area cylinder in sqcm for evaporation / flux area
V=245;%volume cylinder in ccm (does consider holes for tensiometers as 5 ccm)


%no adjustment below this line necessary...
%##########################################












tic;
%INPUT DATA

fid = fopen(headerfile);
headstore = fscanf(fid,'%g\n',[1 6]);
fclose(fid);

fid = fopen(datafile);
filestore = fscanf(fid,'%g %g %g\n',[3 inf]);
filestore = filestore';
fclose(fid);

equipmass=headstore(1);%mass of equipment in gramms
soildry=headstore(2);%mass of soil dry in gramms
W_res1=headstore(3);%residual water in percent after kupf treatment (+/- pf 2.)
W_res2=headstore(4);%residual water in percent after 5 bar (+/- pf 3.)
W_res3=headstore(5);%residual water in percent after 15 bar (+/- pf 4.2)
W_res=headstore(6);%residual water in percent from rosetta


ttens=filestore(:,1);%define top tensiometer value from file
btens=filestore(:,2);%define bottom tensiometer value from file
mass=filestore(:,3);%define total mass from file

pF=log10((abs(ttens)+abs(btens))/2);%calculate actual pF from mean pressure at tensions
water=100*(mass-(equipmass+soildry))/V;%calculate water content in % vol.

%truncate erroneous values after reaching pF>1.8
cut=find(pF>1.8 | pF==max(pF),1);
pF=pF(1:cut);
water=water(1:cut);

pF(length(pF)+1)=3.;
pF(length(pF)+1)=4.2;
pF(length(pF)+1)=7.;
water(length(water)+1)=W_res2;
water(length(water)+1)=W_res3;
water(length(water)+1)=W_res;
theta_sat=water(2);
theta_res=W_res;

figure(1);
hold off;
scatter(pF,water,3)
hold on;


%####################################
%CALCULATION

%generation of initial dummy function
alpha=alpha_ini;
n=n_ini;
pF_dummy=0.0001;

for i=2:100
    pF_dummy(i)=pF_dummy(i-1)+(5/100);
    m=1-1/n;
    h=10^pF_dummy(i);
    theta(i) = theta_res + (theta_sat-theta_res)/(1+(alpha*h)^n)^m;
end
plot(pF_dummy,theta)


%curve fitting
alpha=alpha_ini;
n=n_ini;
errors=ones(3)*(10*fit);
errorss=10000000;
errorss2=errorss;
errorss3=errorss;
counter=0;

while errorss>fit
    counter=counter+1;
    
        nn(1)=n+tuningn;
        nn(2)=n;
        nn(3)=n-tuningn;
        alphan(1)=alpha+tuninga;
        alphan(2)=alpha;
        alphan(3)=alpha-tuninga;
        pFd=0;
        
    for i=1:100
        pFx(i)=pFd+(5/100);
        pFd=pFx(i);
        for j=1:3
            for k=1:3                
                m=1-1/nn(j);
                h=10^pFd;
                theta(i,j,k) = theta_res + ((theta_sat-theta_res)/((1+(alphan(k)*h)^nn(j))^m));
                
                %error calculation as equal steps over all pF
                if find(pF == pFd)
                    pFx=pF(find(pF == pFd));
                    er(i,j,k)=(theta(i,j,k)-water(pFx))^2;
                else
                    a=find(pF < pFd); %find all smaller entries in data
                    aa=0;
                    for l=1:length(a) %identify the greatest of the smaller entries
                        aa=max([aa pF(a(l))]);
                    end
                    aaa=find(pF == aa,1);

                    b=find(pF > pFd); %find all greater entries in data
                    bb=7;
                    for l=1:length(b) %identify the smallest of the greater entries
                        bb=min([bb pF(b(l))]);
                    end
                    bbb=find(pF == bb,1);
                    er(i,j,k)=(theta(i,j,k)-mean([water(aaa) water(bbb)]))^2; %use mean of neighboring values for error calculation
                end
                
                %error calculation with weights depending on pF
%                 eract=(theta(i)-water(i))^2;
%                 er(i,j,k)=eract*(2^abs(pF(i)));%weigh errors to emancipate singular points aof pF 3.7 and 4.2
            end
        end
        thetam(i)=mean(mean(mean(theta(i,j,k))));
    end
    
    for j=1:3
        for k=1:3
            errors(j,k)=sum(er(:,j,k));
        end
    end
    
    for j=1:3
        for k=1:3
            if errors(j,k)==min(min(errors)) && errors(j,k)<errorss
                n=nn(j);
                alpha=alphan(k);
                errorss2=errorss; %memory of last sum of squared errors
                errorss3=errorss2;
                errorss=errors(j,k);
            elseif min(min(errors)) > errorss || abs(errorss3-(min(min(errors)))) < fit2 %break after 3 times no further convergence or actual divergence
                disp('no further convergence')
                disp(['no. of iterations so far:',num2str(counter)])
                %disp(['residual squared errors:',num2str(mean(errors))])
                errorss=1;
                break
            end
        end
    end
    pFx(length(pFx+1))=7;
    thetam(length(thetam+1))=0;
    plot(pFx,thetam)
end


%plot result
%disp('alpha    n')
%disp([alpha n])
pF_dummy=0;

for i=2:100
    pF_dummy(i)=pF_dummy(i-1)+(5/100);
    m=1-1/n;
    h=10^pF_dummy(i);
    thetan(i) = theta_res + (theta_sat-theta_res)/(1+(alpha*h)^n)^m;
end


%plot result
figure(2);
hold off;
scatter(pF,water,4)
hold on;
xvec=pF_dummy;
yvec=thetan;
plot(xvec, yvec,'Color','red','MarkerSize',7,'LineWidth',1)
xlabel(['pF value       -      [est. residual squared error:',num2str(mean(mean(errors))),']']);
ylabel('Theta in vol. %');
title(['fitted van Genuchten Equation with alpha=',num2str(alpha),' and n=',num2str(n),' after ',num2str(counter),' iterations.']);


toc
disp('Done. You might now wish to export the plot:');
disp('e.g. copy/paste this command: print -depsc -tiff ku_pf_out');
disp(['The fitted parameters are alpha=',num2str(alpha),' and n=',num2str(n),' after ',num2str(counter),' iterations.']);

